//---------------------------------------------------------------------------
// This software is released under GNP General Public License (GPL)
// written by Wang Da Qing


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

//---------------------------------------------------------------------------

const long MAXBUFSIZE = 512*1024L;

FILE *fileopen(const char *filename,const char *mode);
long filesize(FILE *stream);
int dump(FILE *source,FILE *dest,unsigned long size);

//---------------------------------------------------------------------------

int main(int argc,char *argv[])
{
	FILE *fp1,*fp2;
	long begin,end,sourcesize;
        long disksize=0,sizeunit=1024;
	char *parm[4];
	float arg4;
	int n,argnum=0;

	if( argc<4 ) {
		printf("Program split file into a small file. writen by WDQ (c) 1993,1994,2004 \n"
		       "       GPL free software            ===== 10/17/2004 =====\n"
		       "Usage: SPLIT filename destname begin(K) [ end(K) ] \n"
		       "   or: SPLIT filename destname begin(B) [ end(B) ] [ /B ]\n"
		       "   or: SPLIT filename destname begin(M) [ end(M) ] [ /M ]\n"
		       "   or: SPLIT filename destname disktype diskno \n"
                       "\n"
		       "       Default use KB as size unit, /B use byte as unit, /M use MB as unit\n"
		       "       Here KB = 1024 bytes, MB = 1,048,576 bytes\n"
		       "       Disktype is 1.44M, 1.2M, 360K and 720K (1.44M FD = 1457664 bytes)\n"
		       "       Use COPY /b file1 + file2 + file3 [destination]\n"
                       "       to merge file.\n"
                       "\n"
		       "For example: if you split a big file  test.zip  into 3 files:\n"
		       "                  test.zip.1, test.zip.2, test.zip.3\n"
		       "         you may use: \n"
                       "                 COPY /b test.zip.1 + test.zip.2 + test.zip.3\n"
                       "         to merge file into  test.zip.1  then rename it to test.zip\n"
                       "         or use\n"
                       "                 COPY /b test.zip.1 + test.zip.2 + test.zip.3 test.zip\n"
                       "         to merge file one step\n"
		       "         ......    Attention: in DOS  COPY must use /b switch at first!\n"
                        );
		return 3;
	 }


        // check command line switch
	for(int i=1;i<argc;i++) {
		n = argv[i][0];
		if( n=='/' || n=='-' ) {
			n= argv[i][1];
                        if( n=='b' || n== 'B')
				sizeunit = 1;
                        else if( n=='m' || n== 'M')
				sizeunit = 1024*1024L;
		 }
		else {
			if(argnum>=4) break;
			parm[argnum++] = argv[i];
		 }
	 }

        // check does destine filename is digital or not
	if( atof(parm[1])!=0 || parm[1][0]=='0' ) {
		printf("Destine filename is %s, may be error.continue [Y/N]? ",parm[1]);
		n=getchar();
		printf("\n");
		if( n!='y' && n!='Y' )
                        return 0;
	 }

	fp1=fileopen(parm[0],"rb");
	sourcesize=filesize(fp1);

        // Floppy Disk type and size
	if( !stricmp(parm[2],"1.44M") )      disksize=(long)1423.5*1024;
	else if( !stricmp(parm[2],"1.2M") )  disksize=(long)1185.5*1024;
	else if( !stricmp(parm[2],"720K") )  disksize=(long)713*1024;
	else if( !stricmp(parm[2],"360K") )  disksize=(long)354*1024;

	if( disksize!=0 && sizeunit!=1024 ) {
		printf("Cannot use /B or /M switch with Diskette Type!\n");
		return 3;
	  }

        // calculate begin and end position
	if( argnum>3 ) arg4=atof(parm[3]);
        else arg4=1;
	if(disksize!=0) {
		begin=((int)arg4-1)*disksize;
		end=(int)arg4*disksize;
	 }
	else {
		begin = atol(parm[2])*sizeunit;
		if(argnum==3) end = sourcesize* sizeunit;     // end position not specified
		else end = arg4*sizeunit;
         }

        // check does position has error
	if( end>sourcesize ) end=sourcesize;
	if( disksize!=0 && ( begin<0 || end<0 || begin>=end ) ) {
		printf("Insufficent diskno!\n");
		return 3;
	   }
	if( begin<0 ) { printf(" Begin must >0 !\n"); return 3; }
	if( end<0 ) { printf(" End must >0 !\n"); return 3; }
	if( begin>=end ) { printf(" Begin must < end !\n"); return 3; }

	fp2=fileopen(parm[1],"wb");
	fseek(fp1,begin,SEEK_SET);

	n=dump(fp1,fp2,end-begin);
	if(n==1) printf("File read error!\n");
	else if(n==2) printf("File write error!\n");
	else {
                printf("File split OK!\n");
		if(end==sourcesize)
                        printf("This is the last part of the file!\n");
	  }

	return 0;
}

int dump(FILE *source,FILE *dest,unsigned long size)
{
	void *buffer;
	unsigned long rsize,msize = MAXBUFSIZE;
	size_t n;

	if( msize>size ) msize = size;
	buffer = malloc(msize);
	if( buffer==NULL ) {
                printf("malloc failed!\n");
                exit(3);
         }

	do {
                if( size>msize )
                        rsize = msize;
                else
                        rsize = size;
                size -= rsize;

		n=fread(buffer,rsize,1,source);
		if(n!=1) return 1;
		n=fwrite(buffer,rsize,1,dest);
		if(n!=1) return 2;
	  } while( size>0 );

	free(buffer);

	fclose(source);
	fclose(dest);

	return 0;
}

FILE *fileopen(const char *filename,const char *mode)
{
	FILE *fp;

	if( (fp=fopen(filename,mode))==NULL ) {
		printf("Can't open file %s.\n",filename);
		exit(3);
	  }
	return fp;
}

long filesize(FILE *stream)
{
   long curpos, length;

   curpos = ftell(stream);
   fseek(stream, 0L, SEEK_END);
   length = ftell(stream);
   fseek(stream, curpos, SEEK_SET);
   return length;
}
